component {

	public any function init( required String directory, Boolean recursive = false ) {
		variables.recursive = arguments.recursive;
		variables.watcher = CreateObject( "java", "java.nio.file.FileSystems" ).getDefault().newWatchService();
		variables.events = CreateObject( "java", "java.nio.file.StandardWatchEventKinds" );
		variables.pathkeys =  CreateObject( "java", "java.util.HashMap" ).init();
		
		var file = CreateObject( "java", "java.io.File" ).init( arguments.directory );
		if ( !file.isDirectory() ) {
			throw( "Directory '#arguments.directory#' does not exist or is not a directory" );
		}
		
		register(file.toPath(), arguments.recursive);
		return this;
	}

	private void function register( required any path, required Boolean recursive ) {
		//writelog( text="register(#arguments.path#, #arguments.recursive#)", file="DirectoryWatcher" );
		var key = arguments.path.register( variables.watcher, [
			variables.events.ENTRY_CREATE,
			variables.events.ENTRY_MODIFY,
			variables.events.ENTRY_DELETE
		]);
		variables.pathKeys.put( key, arguments.path );
		if ( arguments.recursive ) {
			var files = arguments.path.toFile().listFiles();
			for ( var file in files ) {
				if ( file.isDirectory() ) {
					register( file.toPath(), arguments.recursive );
				}
			}
		}
	}

	public any function poll() {
		//writeLog( text="WatchService.Poll()", file="DirectoryWatcher" );
		var _poll = variables.watcher.poll();
		if ( IsNull (_poll) ){
			return [];
		} else {
			return handleEvents( _poll );
		}
	}

	public void function close() {
		//writeLog( text="WatchService.close()", file="DirectoryWatcher" );
		variables.watcher.close();
		variables.watcher = false;
	}

	private function utcTimeToDate(required numeric javaFileLastModified) {
		return dateAdd( "l", arguments.javaFileLastModified, 25569 );
	}

	private Array function handleEvents( required any key ) {
		//writeLog( text="WatchService.handleEvents", file="DirectoryWatcher" );
		var changes = [];
		var path = variables.pathKeys.get( arguments.key );
		if ( !IsNull( path ) ) {
			for ( var event in arguments.key.pollEvents() ) {
				var kind = event.kind();
				//writeLog( text="#Serialize(kind)#", type="error");
				if ( kind !== variables.events.OVERFLOW ) {
					var relativePath = event.context();
					// in the case of ENTRY_CREATE, ENTRY_DELETE, and ENTRY_MODIFY events the context is a Path
					// that is the relative path between the directory registered with the watch service, and the entry that is created, deleted, or modified.
					var affectedPath = path.resolve( relativePath );
					var file = affectedPath.toFile();

					if ( variables.recursive && kind === variables.events.ENTRY_CREATE ) {
						if ( file.isDirectory() ) {
							register( affectedPath, variables.recursive );
						}
					}
					var action = "";
					if ( kind === variables.events.ENTRY_CREATE )
						action = "add";
					else if ( kind === variables.events.ENTRY_DELETE )
						action = "delete";
					else if ( kind === variables.events.ENTRY_MODIFY )
						action = "change";
					else
						throw "Unsupported event: [#kind.name()#]";

					changes.append( {
						"action" = action,
						"name" = file.getName(),
						"size" = file.length(),
						"dateLastModified" = utcTimeToDate( file.lastModified() ),
						"directory" = GetDirectoryFromPath( file.getAbsolutePath() )
					} );
				}
			}
		}
		return changes;
	}

}